pub struct Fallback {
pub enable: bool,
pub code: StatusCode,
pub file: Option<String>,
pub not_found: Option<String>,
}Fields
enable: boolBy default when enabled, returns a prebaked 404 not found page optimized for development. For production set something else (see fields below)
code: StatusCodeFor the unlikely reason to return something different than 404, you
can set it here
file: Option<String>Returns content from a file pointed to by this field with a 404 status
code.
not_found: Option<String>Returns a “404 not found” with a single message string. This sets the message.