pub struct SecureHeader {
pub enable: bool,
pub preset: String,
pub overrides: Option<BTreeMap<String, String>>,
}Expand description
Sets a predefined or custom set of secure headers.
We recommend our github preset. Presets values are derived
from the secure_headers Ruby
library which Github (and originally Twitter) use.
To use a preset, in your config/development.yaml:
middlewares:
secure_headers:
preset: githubYou can also override individual headers on a given preset:
middlewares:
secure_headers:
preset: github
overrides:
foo: barOr start from scratch:
middlewares:
secure_headers:
preset: empty
overrides:
one: twoTo support htmx, You can add the following override, to allow some inline
running of scripts:
secure_headers:
preset: github
overrides:
# this allows you to use HTMX, and has unsafe-inline. Remove or consider in production
"Content-Security-Policy": "default-src 'self' https:; font-src 'self' https: data:; img-src 'self' https: data:; object-src 'none'; script-src 'unsafe-inline' 'self' https:; style-src 'self' https: 'unsafe-inline'"For the list of presets and their content look at secure_headers.json
Fields
enable: boolpreset: Stringoverrides: Option<BTreeMap<String, String>>Trait Implementations
Sourceimpl Clone for SecureHeader
impl Clone for SecureHeader
Sourcefn clone(&self) -> SecureHeader
fn clone(&self) -> SecureHeader
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for SecureHeader
impl Debug for SecureHeader
Sourceimpl Default for SecureHeader
impl Default for SecureHeader
Sourceimpl<'de> Deserialize<'de> for SecureHeader
impl<'de> Deserialize<'de> for SecureHeader
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Sourceimpl MiddlewareLayer for SecureHeader
impl MiddlewareLayer for SecureHeader
Sourcefn is_enabled(&self) -> bool
fn is_enabled(&self) -> bool
Returns whether the middleware is enabled or not
Sourcefn apply(&self, app: AXRouter<AppContext>) -> Result<AXRouter<AppContext>>
fn apply(&self, app: AXRouter<AppContext>) -> Result<AXRouter<AppContext>>
Applies the secure headers layer to the application router