pub struct ModelValidationErrors(/* private fields */);Expand description
Convert ModelValidationErrors (pretty) into a DbErr (ugly) for database
handling.
Because DbErr is used in model hooks and we implement the hooks
in the trait, we MUST use DbErr, so we need to “hide” a representation
of the error in DbErr::Custom, so that it can be unpacked later down the
stream, in the central error response handler.
Methods from Deref<Target = ValidationErrors>
pub fn merge_self( &mut self, field: &'static str, child: Result<(), ValidationErrors>, ) -> &mut ValidationErrors
Sourcepub fn errors(&self) -> &HashMap<Cow<'static, str>, ValidationErrorsKind>
pub fn errors(&self) -> &HashMap<Cow<'static, str>, ValidationErrorsKind>
Returns a map of field-level validation errors found for the struct that was validated and any of it’s nested structs that are tagged for validation.
Sourcepub fn errors_mut(
&mut self,
) -> &mut HashMap<Cow<'static, str>, ValidationErrorsKind>
pub fn errors_mut( &mut self, ) -> &mut HashMap<Cow<'static, str>, ValidationErrorsKind>
Returns a mutable map of field-level validation errors found for the struct that was validated and any of it’s nested structs that are tagged for validation.
Sourcepub fn field_errors(&self) -> HashMap<Cow<'static, str>, &Vec<ValidationError>>
pub fn field_errors(&self) -> HashMap<Cow<'static, str>, &Vec<ValidationError>>
Returns a map of only field-level validation errors found for the struct that was validated.
pub fn add(&mut self, field: &'static str, error: ValidationError)
pub fn is_empty(&self) -> bool